<%-- $Id: failures.jsp 6047 2012-04-18 09:37:46Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.NestedIterator"
	import="net.sf.basedb.util.overview.GenericOverview"
	import="net.sf.basedb.util.overview.OverviewUtil"
	import="net.sf.basedb.util.overview.Node"
	import="net.sf.basedb.util.overview.ValidationOptions"
	import="net.sf.basedb.util.overview.Validator"
	import="net.sf.basedb.util.overview.Severity"
	import="net.sf.basedb.util.overview.Failure"
	import="net.sf.basedb.util.overview.extensions.ValidationRuleAction"
	import="java.util.List"
	import="java.util.LinkedList"
	import="java.util.Iterator"
	import="java.util.Set"
	import="java.util.HashSet"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = null;
try
{
	GenericOverview overview = OverviewUtil.getCurrentOverview(sc);
	ValidationOptions options = overview.getValidationOptions();
	Boolean showFailures = Values.getBoolean(request.getParameter("show_failures"), false);
	List<Failure> failures = overview.getFailures();
	if (failures == null) failures = new LinkedList<Failure>();
	List<Failure> errors = new LinkedList<Failure>();
	List<Failure> warnings = new LinkedList<Failure>();
	Set<Failure> duplicates = new HashSet<Failure>();
	
	for (Failure failure : failures)
	{
		if (!duplicates.contains(failure))
		{
			duplicates.add(failure);
			ValidationRuleAction validator = failure.getValidator();
			Severity severity = options.getSeverity(validator);
			if (severity == Severity.ERROR)
			{
				errors.add(failure);
			}
			else if (severity == Severity.WARNING)
			{
				warnings.add(failure);
			}
		}
	}
	%>
	<base:page title="" type="iframe">
	<base:head scripts="" styles="newjoust.css">
	<script language="JavaScript">
	var selectedFailure = null;
	function failureOnClick(evt, failureId, nodeId, itemType, itemId)
	{
		if (itemType && evt && (evt.altKey || evt.ctrlKey || evt.shiftKey))
		{
			Main.viewOrEditItem('<%=ID%>', itemType, itemId, true);
		}
		else
		{
			window.parent.frames['tree'].selectNode(nodeId);
			if (selectedFailure) Main.removeClass(selectedFailure, 'selected');
			selectedFailure = document.getElementById(failureId);
			Main.addClass(selectedFailure, 'selected');
		}
	}
	function itemOnClick(evt, itemType, itemId)
	{
		Table.itemOnClick(formId, evt, itemId, 'default', viewItem, editItem, returnSelected);
	}
	</script>
	<style>
	tr.failure
	{
		border-top: 1px dotted #A0A0A0; 
		cursor: pointer;
	}
	tr.failure:first-child
	{
		border-top: 1px dotted transparent;
	} 
	tr.failure td 
	{
		vertical-align: bottom;
		padding: 2px;
	}
	tr.failure td.text
	{
		white-space: nowrap;
		overflow: hidden;
		text-overflow: ellipsis;
	}
	.index 
	{
		font-weight: bold;
		text-align: right;
	}

	</style>
	</base:head>

	<base:body>
		<div class="absolutefull" onmousemove="parent.DragDrop.drag(event)" onmouseup="parent.DragDrop.endDrag(event)" >
		
			<div class="absolutefull filled bottomborder" style="height: 2em; padding-left: 1em;">
			<table style="width: 100%; height: 100%;"><tr><td><b>
				<%
				if (showFailures)
				{
					%>
					<%=errors.size() %> error(s),
					<%=warnings.size()%> warning(s)
				<%
				}				
				else
				{
					%>
					<i>Item-tree not validated</i>
					<%
				}
				%>
			</b>
			</td></tr></table>
			</div>
			<%
			if(showFailures)
			{
				%>
				<div class="absolutefull" style="top: 2em; margin-top: 1px;">
				<table style="width: 100%; border-collapse: collapse;">
				<%
				int index = 0;
				Iterator<Failure> all = new NestedIterator<Failure>(errors, warnings);
				while (all.hasNext())
				{
					Failure failure = all.next();
					ValidationRuleAction validator = failure.getValidator();
					Severity severity = options.getSeverity(validator);
					if (severity != Severity.IGNORE)
					{
						index++;
						String failureId = "F" + System.identityHashCode(failure);
						Node node = failure.getNode();
						BasicItem item = node.getItem();
						String onClick = "";
						String tooltip = "";
						if (item == null)
						{
							tooltip = "Get more information about this failure";
							onClick = "failureOnClick(event, '"+failureId+"', '" + node.getId() + "')";
						}
						else
						{
							tooltip = "Get more information about this failure (use CTRL, ALT or SHIFT to edit the item)";
							onClick = "failureOnClick(event, '"+failureId+"', '" + node.getId() + 
								"', '" + item.getType().name() + "', " + item.getId() + ")";
						}
						%>
						<tr id="<%=failureId%>" class="failure highlight" 
							onclick="<%=onClick%>" 
							title="<%=tooltip%>"
							>
							<td style="width: 3em;" class="index"><%=index%></td>
							<td style="width: 24px; text-align: center;" class="text"><base:icon image="<%=severity == Severity.ERROR ? "error.png" : "warning.png"%>" /></td>
							<td class="text link"><%=HTML.encodeTags(failure.getMessage()) %></td>
							<td class="text"><%=HTML.encodeTags(node.getTitle()) %></td>
						</tr>
						<%
					}
				}
				%>
				</table>
				</div>
				<%
			}
			%>
			</div>
		</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
