/* $Id$
	------------------------------------------------------------------
	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for working with subtypable items

	@author Nicklas
*/

var ItemSubtype = new ItemSubtypeClass();
function ItemSubtypeClass()
{
	
	this.cache = new Array();
	
	/**
		Get the subtype of the given item.
		@param The type of the item 
		@param The id of the item
	 */
	this.getSubtype = function(itemType, itemId)
	{
		var request = Ajax.getXmlHttpRequest();
		var ID = getSessionId();
		var url = getRoot() + 'admin/itemsubtypes/ajax.jsp?ID=' + ID;
		url += '&cmd=GetSubtypeForItem&item_id=' + itemId;
		url += '&itemType='+itemType;
		request.open("GET", url, false); 
		request.send(null);
		var response = JSON.parse(request.responseText);
		return response && response.id ? response : null;
	}
	
	/**
		Get the subtype that is related to the given one.
		@param subtypeId The main subtype
		@param relatedItemType The item type of the related subtype we are
			looking for
		@param defaultSubtype The id of the default subtype to return if
			no related subtype is found (optional)
		@return An object with the 'id' and 'name' of the related subtype or null
			if no related subtype is found
	*/
	this.getRelatedSubtype = function(subtypeId, relatedItemType, defaultSubtypeId)
	{
		if (!subtypeId && !defaultSubtypeId) 
		{
			return null;
		}
		
		// Use cached information if we have it
		var info = this.cache['ID'+subtypeId];
		if (info && info[relatedItemType] && info[relatedItemType].related)
		{
			return info[relatedItemType].related;
		}
		
		var request = Ajax.getXmlHttpRequest();
		var ID = getSessionId();
		var url = getRoot() + 'admin/itemsubtypes/ajax.jsp?ID=' + ID;
		url += "&cmd=GetRelatedSubtype&relatedType=" + relatedItemType;
		if (subtypeId) 
		{
			url += '&item_id='+subtypeId;
		}
		if (defaultSubtypeId) 
		{
			url += '&defaultRelatedId=' + defaultSubtypeId;
		}
		request.open("GET", url, false); 
		request.send(null);
		
		var response = JSON.parse(request.responseText);
		return response && response.id ? response : null;
	}

	this.createRelatedFilter = function(formId, relatedItemType, defaultSubtypeId, listName)
	{
		var subtypeId = this.getSubtypeId(formId, listName);
		var relatedSubtype = this.getRelatedSubtype(subtypeId, relatedItemType, defaultSubtypeId);
		return relatedSubtype ? '&tmpfilter:INT:itemSubtype=' + relatedSubtype.id : '';
	}
	
	this.getSubtypeId = function(formId, listName)
	{
		var frm = document.forms[formId];
		if (!frm) return null;
		if (!listName) listName = 'subtype_id';
		var list = frm[listName];
		if (!list || list.disabled || list.length == 0) return null;
		return Math.abs(parseInt(list[list.selectedIndex].value));
	}
	
	this.getRecentAndDefaultInfo = function(subtypeId, mainItemType)
	{
		var info = this.cache['ID' + subtypeId];
		if (info) return info;

		var request = Ajax.getXmlHttpRequest();
		var url = getRoot() + 'admin/itemsubtypes/ajax.jsp?ID=' + getSessionId();
		url += '&cmd=GetRecentAndDefault';
		url += '&item_id='+subtypeId;
		if (mainItemType) url += '&itemType='+mainItemType;
		request.open("GET", url, false); 
		request.send(null);
		info = JSON.parse(request.responseText);
		this.cache['ID'+subtypeId] = info;
		return info;
	}
	
	this.getRecentAndRelatedInfo = function(subtypeId, mainItemType, relatedItemTypes)
	{
		var info = this.cache['ID' + subtypeId];
		if (info)
		{
			// We have some info already, but...
			// ...check if we are missing info about some related item types
			var missingInfo = new Array();
			for (var i = 0; i < relatedItemTypes.length; i++)
			{
				if (!info[relatedItemTypes[i]]) missingInfo[missingInfo.length] = relatedItemTypes[i];
			}
			relatedItemTypes = missingInfo;
		}
		
		// Get more info...
		if (relatedItemTypes.length > 0)
		{
			var request = Ajax.getXmlHttpRequest();
			var url = getRoot() + 'admin/itemsubtypes/ajax.jsp?ID=' + getSessionId();
			url += '&cmd=GetRecentAndRelated&itemType='+mainItemType;
			if (subtypeId) 
			{
				url += '&item_id='+subtypeId;
			}
			for (var i = 0; i < relatedItemTypes.length; i++)
			{
				url += '&relatedType=' + relatedItemTypes[i];
			}
			request.open("GET", url, false); 
			request.send(null);
			var moreInfo = JSON.parse(request.responseText);
			if (!info)
			{
				info = moreInfo;
			}
			else
			{
				// Merge the new info with the existing info
				for (var i = 0; i < relatedItemTypes.length; i++)
				{
					info[relatedItemTypes[i]]= moreInfo[relatedItemTypes[i]];
				}
			}
			this.cache['ID'+subtypeId] = info;
		}
		return info;
	}
	
	/**
		Update the selection list with 'recently used' and 'project default' items.
		Currently existing options under those headers will be cleared. 
	*/
	this.updateSelectionList = function(list, recentItems, projectDefaults, noNoneOption)
	{
		var oldSelectedValue = list.selectedIndex >= 0 ? list[list.selectedIndex].value : 0;
		var reselectValue = 0;
		
		// Find the first header option
		for (var i = noNoneOption ? 0 : 1; i < list.length; i++)
		{
			if (list[i].value == 0)
			{
				// Found the header entry
				// Remember the currently selected value if it is after header
				if (list.selectedIndex > i)
				{
					reselectValue = list[list.selectedIndex].value;
				}
				// Clear everything from the header and onwards
				list.length = i;
			}
		}
		
		var firstNewIndex = list.length + 1;
		if (recentItems && recentItems.length)
		{
			// Add recently used items
			var recentlyUsedHeader = new Option('- recently used -', 0);
			recentlyUsedHeader.className = 'recentheader';
			recentlyUsedHeader.disabled = true;
			list[list.length] = recentlyUsedHeader;
			
			for (var i = 0; i < recentItems.length; i++)
			{
				var selected = reselectValue == recentItems[i].id;
				list[list.length] = new Option((i+1) + '. ' + recentItems[i].name, recentItems[i].id, false, selected);
			}
		}
		
		if (projectDefaults && projectDefaults.length)
		{
			// Add project default items
			var projectDefaultHeader = new Option('- project default -', 0);
			projectDefaultHeader.className = 'defaultheader';
			projectDefaultHeader.disabled = true;
			list[list.length] = projectDefaultHeader;
			
			for (var i = 0; i < projectDefaults.length; i++)
			{
				var selected = reselectValue == projectDefaults[i].id;
				list[list.length] = new Option( projectDefaults[i].name, projectDefaults[i].id, false, selected);
			}
		}
		
		// If no item is selected, select the first new item
		if (list.selectedIndex == 0 && list.length > firstNewIndex) 
		{
			list.selectedIndex = firstNewIndex;
		}
		
		var currentSelectedValue = list.selectedIndex >= 0 ? list[list.selectedIndex].value : 0;
		return oldSelectedValue != currentSelectedValue;
	}
	
}

