<%-- $Id: settings.jsp 5977 2012-02-20 13:26:26Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	This page is used to display and modify the attributes of
	the logged in user. It displays a tabbed dialogue:

	@param page
		The name active page of the tabbed dialog.
		- contact: Contact information for the user, address, phone, email, etc.
		- password: The password for the user
		- other: Other information

	Saving the form invokes the submit_user.jsp page with cmd=SaveSettings

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.*"
	import="net.sf.basedb.util.EmailUtil"
	import="net.sf.basedb.clients.web.*"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.formatter.Formatter"	
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.formatter.FormatterSettings"
	import="java.util.Date"
	import="java.util.List"		
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String activePage = request.getParameter("page");
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
final ItemContext cc = Base.getAndSetCurrentContext(sc, Item.USER, null, null);
try
{
	final User user = User.getById(dc, sc.getLoggedInUserId());
	final String title = "Information for " + HTML.encodeTags(user.getName());
	final boolean writePermission = user.hasPermission(Permission.RESTRICTED_WRITE);
	final String ifDisabled = writePermission ? "" : "disabled";
	final String passwordDisabled = writePermission && Application.isUsingInternalAuthentication() ? 
		"" : "disabled";
	dc.detachItem(user);
	sc.setSessionSetting("user", user);
	
	List<ExtendedProperty> extendedProperties = ExtendedProperties.getProperties("UserData");
	
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	String dateFormat = FormatterSettings.getDateFormat(sc);
	String jsDateFormat = HTML.javaScriptEncode(dateFormat);
	String htmlDateFormat = HTML.encodeTags(dateFormat);
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	String dateTimeFormat = FormatterSettings.getDateTimeFormat(sc);
	String jsDateTimeFormat = HTML.javaScriptEncode(dateTimeFormat);
	String htmlDateTimeFormat = HTML.encodeTags(dateTimeFormat);
	%>
	<base:page type="popup" title="<%=title%>">
	<base:head scripts="tabcontrol.js" styles="tabcontrol.css">
		<script language="JavaScript">
		// Validate the "Contact information" tab
		function validateContact()
		{
			return true;
		}
		// Validate the "Password" tab - check that both passwords match
		function validatePassword()
		{
			var frm = document.forms["user"];
			if (frm.password.value != frm.retype_password.value)
			{
				frm.retype_password.focus();
				alert("The two passwords do not match!");
				return false;
			} 
			return true;
		}
		
		// Validate the "Extended properties" tab
		function validateExtendedProperties()
		{
			var frm = document.forms['user'];
			var field;
			<%
			if (extendedProperties != null)
			{
				for (ExtendedProperty ep : extendedProperties)
				{
					Type type = ep.getType();
					String name = ep.getName();
					%>
					field = frm['<%=name%>'];
					<%
					if (type == Type.DATE)
					{
						%>
						if (field.value != '' && !Dates.isDate(field.value, '<%=jsDateFormat%>'))
						{
							alert("'"+field.value+"' is not a valid date for '<%=HTML.javaScriptEncode(ep.getTitle())%>'");
							field.focus();
							return false;
						}
						<%
					}
					else if (type == Type.TIMESTAMP)
					{
						%>
						if (field.value != '' && !Dates.isDate(field.value, '<%=jsDateTimeFormat%>'))
						{
							alert("'"+field.value+"' is not a valid timestamp for '<%=HTML.javaScriptEncode(ep.getTitle())%>'");
							field.focus();
							return false;
						}
						<%
					}
					if (!ep.isNullable())
					{
						if (type == Type.BOOLEAN)
						{
							%>
							if (Forms.getCheckedRadio(field) == null)
							{
								alert("You must select a value for '<%=HTML.javaScriptEncode(ep.getTitle())%>'");
								field.focus();
								return false;
							}
							<%
						}
						else
						{
							%>
							if (Main.trimString(field.value) == '')
							{
								alert("You must enter a value for '<%=HTML.javaScriptEncode(ep.getTitle())%>'.");
								field.focus();
								return false;
							}
							<%
						}
					}
				}
			}
			%>
			return true;
		}
		
		// Submit the form
		function saveSettings()
		{
			var frm = document.forms["user"];
			if (TabControl.validateActiveTab('settings'))
			{
				frm.submit();
			}
		}
		</script>
	</base:head>
	<base:body>
		<h1><%=title%><base:help tabcontrol="settings" /></h1>
	
		<form action="submit_user.jsp?ID=<%=ID%>" method="post" name="user" onsubmit="return false;">
		<input type=hidden name="cmd" value="SaveSettings">
	
		<t:tabcontrol 
			subclass="content dialogtabcontrol"
			active="<%=activePage%>" id="settings" 
			position="bottom" remember="false">
	
			<t:tab id="contact" title="Contact information" validate="validateContact()" helpid="userpreferences.contact">
				<table class="fullform input100">
				<tr>
					<th>Full name</th>
					<td><%=HTML.encodeTags(user.getName())%></td>
					<td></td>
				</tr>
				<tr>
					<th>Email</th>
					<td><input <%=ifDisabled%> type="text" name="email" value="<%=HTML.encodeTags(user.getEmail())%>" maxlength="<%=User.MAX_EMAIL_LENGTH%>"></td>
					<td></td>
				</tr>
				<%
				if (EmailUtil.isEnabled() && writePermission)
				{
					%>
					<tr>
						<th class="subprompt"></th>
						<td>
							<%
							boolean sendMessagesAsEmail = user.getSendMessagesAsEmail();
							%>
							<input type="checkbox" name="sendMessagesAsEmail" id="sendMessagesAsEmail" value="1" <%=sendMessagesAsEmail ? "checked" : ""%>
								><label for="sendMessagesAsEmail">Send system messages as email</label>
						</td>
						<td></td>
					</tr>
					<%
				}
				%>
				<tr>
					<th>Organisation</th>
					<td><input <%=ifDisabled%> type="text" name="organisation" value="<%=HTML.encodeTags(user.getOrganisation())%>" maxlength="<%=User.MAX_ORGANISATION_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr class="big">
					<th>Address</th>
					<td>
						<textarea <%=ifDisabled%> rows="4" name="address"><%=HTML.encodeTags(user.getAddress())%></textarea>
					</td>
					<td style="width: 20px;">
						<base:icon image="zoom.png" 
							onclick="Main.zoom('Address', 'user', 'address')" 
							visible="<%=writePermission%>"
							tooltip="Edit in larger window" />
					</td>
				</tr>
				<tr>
					<th>Phone</th>
					<td><input <%=ifDisabled%> type="text" name="phone" value="<%=HTML.encodeTags(user.getPhone())%>" maxlength="<%=User.MAX_PHONE_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr>
					<th>Fax</th>
					<td><input <%=ifDisabled%> type="text" name="fax" value="<%=HTML.encodeTags(user.getFax())%>" maxlength="<%=User.MAX_FAX_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr>
					<th>Url</th>
					<td><input <%=ifDisabled%> type="text" name="url" value="<%=HTML.encodeTags(user.getUrl())%>" maxlength="<%=User.MAX_URL_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr class="dynamic">
					<th></th>
					<td></td>
					<td></td>
				</tr>
				</table>
			</t:tab>
			
			<t:tab id="password" title="Password" validate="validatePassword()" helpid="userpreferences.password">
				<table class="fullform input100">
				<% if (!Application.isUsingInternalAuthentication() && writePermission)
				{
					%>
					<tr>
						<td colspan="2">
						<div class="messagecontainer error">Base is using external authentication. Password cannot be changed here.</div>
						</td>
					</tr>
					<%
				}
				%>
				<tr>
					<th>Login</th>
					<td><%=HTML.encodeTags(user.getLogin())%></td>
				</tr>
				<tr>
					<th>New password</th>
					<td><input <%=passwordDisabled%> type="password" name="password" value=""  maxlength="30"></td>
				</tr>
				<tr>
					<th>Retype password</th>
					<td><input <%=passwordDisabled%> type="password" name="retype_password" value="" maxlength="30"></td>
				</tr>
				<tr class="dynamic">
					<th></th>
					<td><i>- leave empty to not change the password</i></td>
				</tr>
				</table>
			</t:tab>
		
			<t:tab id="other" title="Other information" 
				validate="validateExtendedProperties()" helpid="userpreferences.other">
				<table class="fullform input100">
				<%
				if (extendedProperties != null)
				{
					for (ExtendedProperty ep : ExtendedProperties.getProperties("UserData"))
					{
						String name = ep.getName();
						Type type = ep.getType();
						boolean required = !ep.isNullable();
						Object value = user == null ? cc.getPropertyValue(name) : user.getExtended(name);
						String isRequired = required ? "class=\"required\"" : "";
						boolean addZoom = false;
						%>
						<tr>
							<th><%=HTML.encodeTags(ep.getTitle())%></th>
							<td>
							<%
							if (type == Type.INT || type == Type.LONG)
							{
								%>
								<input <%=isRequired%> <%=ifDisabled %> type="text" name="<%=name%>" 
									value="<%=value == null ? "" : value%>" style="width: 15m;"
									maxlength="20" onkeypress="return Numbers.integerOnly(event)"
									>
								<%
							}
							else if (type == Type.FLOAT || type == Type.DOUBLE)
							{
								%>
								<input <%=isRequired%> <%=ifDisabled%> type="text" name="<%=name%>"
									value="<%=value == null ? "" : value%>" style="width: 15m;"
									maxlength="20" onkeypress="return Numbers.numberOnly(event)"
									>
								<%
							}
							else if (type == Type.STRING)
							{
								%>
								<input <%=isRequired%> <%=ifDisabled%> type="text" name="<%=name%>"
									value="<%=HTML.encodeTags((String)value)%>"
									maxlength="<%=ep.getLength()%>"
									>
								<%
							}
							else if (type == Type.TEXT)
							{
								addZoom = true;
								%>
								<textarea <%=isRequired%> <%=ifDisabled%> name="<%=name%>" rows="6"  
									><%=HTML.encodeTags((String)value)%></textarea>
								<%
							}
							else if (type == Type.BOOLEAN)
							{
								Boolean b = (Boolean)value;
								if (!required)
								{
									%>
									<input type="radio" name="<%=name%>" id="<%=name%>.null" value=""
										<%=b == null ? "checked" : ""%> 
										><label for="<%=name%>.null"><i>- not specified -</i></label><br>
									<%
								}
								%>
								<input type="radio" name="<%=name%>" id="<%=name%>.true" value="true" 
									<%=b != null && b == true ? "checked" : ""%>
									><label for="<%=name%>.true">true</label><br>
								<input type="radio" name="<%=name%>" id="<%=name%>.false" value="false" 
									<%=b != null && b == false ? "checked" : ""%>
									><label for="<%=name%>.false">false</label>
								<%
							}
							else if (type == Type.DATE)
							{
								%>
								<table>
								<tr>
								<td>
									<input <%=isRequired%> <%=ifDisabled%> type="text" name="<%=name%>" 
										value="<%=dateFormatter.format((Date)value)%>" style="width: 15em;"
										maxlength="20" title="Enter date in format: <%=htmlDateFormat%>"
										>
								</td>
								<td>
									<base:button 
										onclick="<%="Dates.selectDate('"+HTML.javaScriptEncode(ep.getTitle())+"', 'user', '"+name+"', null, '"+jsDateFormat +"')"%>" 
										image="calendar.png"
										title="Calendar&hellip;" 
										tooltip="Select a date from a calendar" 
										visible="<%=writePermission%>"
									/>
								</td>
								</tr>
								</table>
								<%
							}
							else if (type == Type.TIMESTAMP)
							{
								%>
								<table>
								<tr>
								<td>
									<input <%=isRequired%> <%=ifDisabled%> type="text" name="<%=name%>" 
										value="<%=dateTimeFormatter.format((Date)value)%>" style="width: 15em;"
										maxlength="20" title="Enter timestamp in format: <%=htmlDateTimeFormat%>"
										>
								</td>
								<td>
									<base:button 
										onclick="<%="Dates.selectDateTime('"+HTML.javaScriptEncode(ep.getTitle())+"', 'user', '"+name+"', null, '"+jsDateTimeFormat +"')"%>" 
										image="calendar.png"
										title="Calendar&hellip;" 
										tooltip="Select a timestamp from a calendar" 
										visible="<%=writePermission%>"
									/>
								</td>
								</tr>
								</table>
								<%
							}
							%>
							</td>
							<td style="width: 20px;">
								<%if (addZoom)
								{
									%>
									<base:icon image="zoom.png" 
									onclick="<%="Main.zoom('" + HTML.javaScriptEncode(ep.getTitle()) + "', 'user', '" + name + "')"%>"
										tooltip="Edit in larger window"
										visible="<%=writePermission%>"
									/>
									<%
								}
								%>
							</td>
						</tr>
						<%
					}
				}
				%>
				<tr class="dynamic">
					<th>Description</th>
					<td>
						<textarea <%=ifDisabled%> rows="5" name="description"><%=HTML.encodeTags(user.getDescription())%></textarea>
					</td>
					<td style="width: 20px;">
						<base:icon image="zoom.png" 
							onclick="Main.zoom('Description', 'user', 'description')"
							tooltip="Edit in larger window"
							visible="<%=writePermission%>"
						/>
					</td>
				</tr>
				</table>
			</t:tab>
		</t:tabcontrol>
		</form>
	
		<base:buttongroup subclass="dialogbuttons">
			<base:button onclick="saveSettings();" title="Save" visible="<%=writePermission%>"/>
			<base:button onclick="window.close();" title="Cancel" visible="<%=writePermission%>"/>
			<base:button onclick="window.close();" title="Close" visible="<%=!writePermission%>"/>
		</base:buttongroup>
	
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
